% Fuzzy Logic Toolbox Demos.
%
% General demos
%   defuzzdm   - Defuzzification methods.
%   fcmdemo    - Fuzzy c-means clustering demo (2-D).
%   fuzdemos   - List of all Fuzzy Logic Toolbox demos.
%   invkine    - Inverse kinematics system of a robot arm.
%   irisfcm    - Fuzzy c-means clustering demo (4-D).
%   juggler    - Juggling ball system.
%   mfdemo     - Membership functions.
%   mgtsdemo   - Mackey-Glass time series analysis.
%   noisedm    - Active noise cancellation.
%   trips      - Subtractive clustering demo with demographic data.
%
% SIMULINK demos
%   shower     - Shower temperature/flow control model (SIMULINK only).
%   slbb       - Ball and beam control (SIMULINK only)
%   slcp       - Inverted pendulum control (SIMULINK only)
%   slcp1      - Inv. pendulum with varying pole length (SIMULINK only)
%   slcpp1     - Double inv. pendulum with varying pole length (SIMULINK only)
%   sltank     - Water level control (SIMULINK only)
%   sltbu      - Truck backer-upper (SIMULINK only)

% Fuzzy inference system (.fis) files
%   custtip    - Customized model file.
%   invkine1   - Inverse kinematics system 1.
%   invkine2   - Inverse kinematics system 2.
%   juggler    - Juggler system.
%   mam21      - Sample one input two output Mamdani-style system.
%   membrn1    - ANFIS-trained membrane plot (two MFs per input).
%   membrn2    - ANFIS-trained membrane plot (three MFs per input).
%   peaks      - ANFIS-trained peaks plot (three MFs per input).
%   shower     - Shower temperature/flow system.
%   slbb       - Ball and beam system.
%   slcp       - Inverted pendulum system.
%   slcp1      - Inv. pendulum with varying pole length system.
%   slcpp1     - Double inv. pendulum with varying pole length system.
%   sltbu      - Truck backer-upper system.
%   sugeno1    - Simple Sugeno-style system.
%   tank       - Water tank system.
%   tank2      - Improved tank system.
%   tanksg     - Sugeno version of the water tank system.
%   tipper     - Tipping calculation system (two inputs one output).
%   tipper1    - Tipping calculation system (one input one output).
%   tippersg   - Tipping calculation system (Sugeno-style).
%
% Customized FIS support functions
%   custagg    - Customized aggregation function for CUSTTIP.FIS.
%   custand    - Customized AND function for CUSTTIP.FIS.
%   custdfuz   - Customized defuzzification function for CUSTTIP.FIS.
%   custimp    - Customized implication function for CUSTTIP.FIS.
%   custmf     - Customized membership function for CUSTTIP.FIS.  
%   custor     - Customized OR functions for CUSTTIP.FIS.
%
% Helper functions
%   animbb     - Animate ball and beam.
%   animcp     - Animate cart and pole.
%   animcp1    - Animate cart and pole.
%   animcpp1   - Animate cart and parallel poles.
%   animtank   - Animate water tank.
%   animtbu1   - Truck backing animation 1.
%   animtbu2   - Truck backing animation 2.
%   ctrltbu1   - Control truck backer 1.
%   ctrltbu2   - Control truck backer 2.
%   initbb     - Initialize ball and beam demo.
%   initcp     - Initialize cart and pole demo.
%   initcp1    - Initialize cart and pole demo.
%   initcpp1   - Initialize cart and parallel poles.
%   inittbu    - Initialize truck backing demo.
%   invkine    - Inverse kinematics of a robot arm.
%   invsurf    - Plot four surfaces for the robot arm demo.
%   mktipper   - Build tipping model with a script.
%   traininv   - Train ANFIS for inverse modeling of robot arm.
%   uiarray    - Create an array of UI buttons.

%   Copyright 1994-2002 The MathWorks, Inc. 
%   Version 1.0.2  15-Nov-1996  
%   $Revision: 1.10 $  $Date: 2002/04/14 22:17:00 $

